/*
*
* Gtk+1.2-Gtk+2.0  Utility routines
* Copyright (C) 2002-2003-2004 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

/********************* About and license file viewer Widget *******************************/

#include <gtk/gtk.h>
#include <stdlib.h>

#ifdef USE_GTK_2_0
void about_wrap_callback(GtkWidget *widget, GtkWidget *text)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	if(gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(text)) == GTK_WRAP_NONE)
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
	else
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_NONE);
}
#else
void about_wrap_callback(GtkWidget *widget, GtkText *text)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	gtk_text_set_line_wrap ( GTK_TEXT (text), (text->line_wrap == FALSE) ? TRUE : FALSE);
	gtk_text_set_word_wrap ( GTK_TEXT (text), (text->word_wrap == FALSE) ? TRUE : FALSE);
}
#endif

void license_callback(GtkWidget *widget, int *ptr_status)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	SET2(*ptr_status,1);
}

int about_window_create(char * title, int x, int y, int border, int modal_flag, char *labeltext, FILE *fd, int first_time)
{
	GtkWidget		*edit_window;
	GtkWidget		*table;
	GtkWidget		*frame;
	GtkWidget		*label;
	GtkWidget		*text;
	GtkWidget		*bbox;
	GtkWidget		*check_button;
	GtkWidget		*button;
	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		iter;
	#else
	GdkFont		*fixed_font;
	#endif
	char *temp_ptr;
	int status=0;
	int *ptr_status;

	ptr_status=&status;

	DBG_FUNCTION_NAME;

	/* Make the gui first  */
	edit_window = window_create((x<=0)?530:x, (y<=0)?430:y, border, title, gtk_main_quit , &edit_window ,modal_flag);
	table = table_create( 20, 18, edit_window , 0, NOT_HOMOGENEUS);

	#ifdef USE_GTK_2_0
	frame = frame_create_on_table( NULL, table, 0, 20, 0, 4);
	label = label_create_on_table(labeltext, table, 1, 19, 0, 4);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
	text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, table,  0, 20, 4, 16);
	#else
	frame = frame_create_on_table( NULL, table, 0, 20, 0, 5);
	label = label_create_on_table(labeltext, table, 0, 20, 0, 5);
	text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, table,  0, 19, 5, 16);
	#endif

	check_button = check_create_on_table(STR_WRAP, GTK_SIGNAL_FUNC(wrap_callback), text, INACTIVE, table, 0, 3, 16, 17);
	if(first_time==FIRST_TIME)
		check_button = check_create_on_table(STR_I_AGREE_DONT_SHOW_IT_AGAIN, GTK_SIGNAL_FUNC(license_callback), ptr_status,
											 INACTIVE, table, 0, 3, 17, 18);

	bbox = default_button_box_create_on_table( HORIZONTAL, table, 3, 20, 16, 18);
	button = button_create_in_container(STR_OK, gtk_main_quit , edit_window ,bbox);
	gtk_widget_show_all(edit_window);

	#ifdef USE_GTK_2_0
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	#else
	/* Load a fixed font */
	fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-130-*-*-*-*-*-*");
	#endif

	/* Read from fd */
	if(fd==NULL)
	{
		text_view_error(edit_window, modal_flag , STR_CANNOT_OPEN_FILE);
	}
	else
	{
		#ifndef USE_GTK_2_0
		gtk_text_freeze(GTK_TEXT(text));
		#endif
		while((temp_ptr=xfgetc_nbuf(fd, DEF_BUFFER_SIZE))!=NULL && GTK_IS_WIDGET(text) )
		{
			#ifdef USE_GTK_2_0
			if(read_and_convert(temp_ptr, buffer, iter, edit_window, modal_flag)==1)
				break;
			#else
			gtk_text_insert(GTK_TEXT(text), fixed_font, &text->style->black, NULL, temp_ptr, -1);
			#endif
			while (gtk_events_pending ())
				gtk_main_iteration();
			free(temp_ptr);
		}
		#ifndef USE_GTK_2_0
		gtk_text_thaw(GTK_TEXT(text));
		#endif
	}

	gtk_main();
	if(GTK_IS_WIDGET(edit_window))
		gtk_widget_destroy(edit_window);
	return status;
}

/*************** End of  About and license file viewer Widget ********************************/
