/*
*
* Gtk+1.2-Gtk+2.2  Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>

void text_view_error(GtkWidget *window, int modal_flag , char * msg_string)
{
	DBG_FUNCTION_NAME;

	if(modal_flag==MODAL)
		gtk_window_set_modal (GTK_WINDOW(window), FALSE );
	dialog_window(STR_ALERT, msg_string, ONE_BUTTON, MODAL, NULL);
	if(modal_flag==MODAL)
		gtk_window_set_modal (GTK_WINDOW(window), TRUE);
}

FILE *open_file(char *filename )
{
	/*   Inspired by files.c of nano editor 1.2.2                            *
	*                                                                        *
	*   Copyright (C) 1999-2003 Chris Allegretta GPL                         */

	int fd;
	char *fmt;
	FILE *file=NULL;
	struct stat fileinfo;

	DBG_FUNCTION_NAME;

	if (filename[0] == '\0' || stat(filename, &fileinfo) == -1)
	{
		fmt = STR_UNABLE_TO_STAT_S_S;
		error_window(STR_ERROR, fmt, filename);
	}
	else if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) || S_ISBLK(fileinfo.st_mode))
	{
		/* Don't open character or block files.  Sorry, /dev/sndstat! */
		if(S_ISDIR(fileinfo.st_mode))
			fmt=STR_S_IS_A_DIR;
		else
			fmt=STR_FILE_S_IS_A_DEVICE_FILE;

		error_window(STR_ERROR, fmt, filename);
	}
	else if ((fd = open(filename, O_RDONLY)) == -1)
	{
		fmt="%s: %s";
		error_window(STR_ERROR, fmt, filename);
	}
	else
	{
		/* File is A-OK */
		/* Binary for our own line-end munging */
		/*The mode string can also include the letter  ``b''  either  as  a  last
		character  or  as a character between the characters in any of the two-
		character strings described above.  This is strictly for  compatibility
		with  ANSI  X3.159-1989  (``ANSI  C'')  and has no effect; the ``b'' is
		ignored on all POSIX conforming systems, including Linux.*/
		file = fdopen(fd, "rb");
		if (file == NULL)
		{
			fmt= STR_ERROR_READING_FILE_S_S;
			error_window(STR_ERROR, fmt, filename);
			close(fd);
			file=NULL;
		}
	}
	/* end of nano code */
	return file;
}

#ifdef USE_GTK_2_0
int read_and_convert(char * temp_ptr, GtkTextBuffer *buffer, GtkTextIter iter , GtkWidget *window, int modal_flag)
{
	char * utf8_temp=NULL;

	DBG_FUNCTION_NAME;
	/* Revalidate the iterator */
	gtk_text_buffer_get_end_iter(buffer, &iter);

	/* Convert to UTF-8 */
	if((utf8_temp=g_locale_to_utf8(temp_ptr,strlen(temp_ptr),NULL,NULL,NULL))!=NULL)
	{
		gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, utf8_temp, -1, "monospace", NULL);
		free(utf8_temp);
		return (0);
	}
	/*dialog_window("Alert", "Conversion to UTF-8 failed!", ONE_BUTTON, MODAL, NULL);*/
	text_view_error(window, modal_flag ,  STR_CONVERSION_TO_UTF8_FAILED);
	return (1);
}
#endif

