/*
*
* Gtk+1.2-Gtk+2.2  Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <strings.h>

/********************* Text file edit Widget *******************************/

void save_file(char *filename, GtkWidget *text)
{
	/*   Inspired by files.c of nano editor 1.2.2                            *
	*                                                                        *
	*   Copyright (C) 1999-2003 Chris Allegretta GPL                         *
	*   realeased under the terms of the GPL                                 */

	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		start;
	GtkTextIter		end;
	#endif
	char *data=NULL;
	int flag;
	int fd;
	char *fmt;
	FILE *file=NULL;
	struct stat fileinfo;

	#ifdef DEBUG
	err_msg("save_file(): %s",filename);
	#endif

	if (filename[0] == '\0' || filename == NULL )
	{
		error_window(NULL, STR_NO_FILE_NAME, "");
		return;
	}

	if (stat(filename, &fileinfo) == 0)
	{
		/* File exists */
		if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) || S_ISBLK(fileinfo.st_mode))
		{
			if(S_ISDIR(fileinfo.st_mode))
				fmt=STR_S_IS_A_DIR;
			else
				fmt=STR_FILE_S_IS_A_DEVICE_FILE;

			error_window(NULL, fmt, filename);
			return;
		}
		flag=O_WRONLY|  O_TRUNC ;
	}
	else
	{
		flag= O_WRONLY| O_CREAT;
	}

	if ((fd = open(filename,  flag )) == -1)
	{
		fmt="%s: %s";
		error_window(NULL,fmt, filename);
		return;
	}

	/* File is A-OK */
	/*The mode string can also include the letter  ``b''  either  as  a  last
	character  or  as a character between the characters in any of the two-
	character strings described above.  This is strictly for  compatibility
	with  ANSI  X3.159-1989  (``ANSI  C'')  and has no effect; the ``b'' is
	ignored on all POSIX conforming systems, including Linux.*/
	if ((file = fdopen(fd, "wb"))== NULL)
	{
		fmt=STR_ERROR_WRITING_FILE_S_S;
		error_window(NULL, fmt, filename);
		close(fd);
		return;
	}
	#ifdef USE_GTK_2_0
	buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	data=g_locale_from_utf8(gtk_text_buffer_get_text(buffer, &start, &end,/*include_hidden_chars*/ FALSE), -1, NULL, NULL, NULL);
	#else
	data=gtk_editable_get_chars( GTK_EDITABLE(text) , 0, -1);
	#endif
	if(data==NULL)
	{
		error_window(NULL, STR_CANNOT_SAVE_S_UNKNOWN_ERROR, filename);
		fclose(file);
		return;
	}
	fprintf(file,"%s", data);
	free(data);
	fclose(file);
}


void text_changed_callback(GtkWidget *widget, int *ptr_text_modified)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	SET(*ptr_text_modified);
}

void about_callback(void)
{
	DBG_FUNCTION_NAME;
	dialog_window(STR_ABOUT, VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY, ONE_BUTTON, MODAL, NULL);
}

void text_edit_menu_callback(unsigned int * action, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	SET2(*action, callback_action);
	gtk_main_quit();
}

#ifdef USE_GTK_2_0
static GtkItemFactoryEntry text_edit_menu_items[] = {
	{ MNU_FILE,					NULL,				NULL,								0,	"<Branch>",		NULL},
	{ MNU_FILE_NEW,				SHC_FILE_NEW,		text_edit_menu_callback,			1,	NULL,			NULL},
	{ MNU_FILE_OPEN,			SHC_FILE_OPEN,		text_edit_menu_callback,			2,	NULL,			NULL},
	{ MNU_FILE_SAVE,			SHC_FILE_SAVE,		text_edit_menu_callback,			3,	NULL,			NULL},
	{ MNU_FILE_SAVE_AS,			SHC_FILE_SAVE_AS,	text_edit_menu_callback,			4,	NULL,			NULL},
	{ MNU_FILE_QUIT,			SHC_FILE_QUIT,		text_edit_menu_callback,			6,	NULL,			NULL},
	{ MNU_OPTIONS,				NULL,				NULL,								0,	"<Branch>",		NULL},
	{ MNU_OPTIONS_WRAP,			SHC_OPTIONS_WRAP,	text_edit_menu_callback,			9,	"<CheckItem>",	NULL},
	{ MNU_HELP,					NULL,				NULL,								0,	"<LastBranch>",	NULL},
	{ MNU_HELP_ABOUT,			NULL,				text_edit_menu_callback,			11,	NULL,			NULL},
	{ NULL,						NULL,				NULL,								0,	NULL,			NULL},
	 /* Set last item to NULL */
};
#else
static GtkItemFactoryEntry text_edit_menu_items[] = {
	{ MNU_FILE,					NULL,				NULL,								0,	"<Branch>"		},
	{ MNU_FILE_NEW,				SHC_FILE_NEW,		text_edit_menu_callback,			1,	NULL			},
	{ MNU_FILE_OPEN,			SHC_FILE_OPEN,		text_edit_menu_callback,			2,	NULL			},
	{ MNU_FILE_SAVE,			SHC_FILE_SAVE,		text_edit_menu_callback,			3,	NULL			},
	{ MNU_FILE_SAVE_AS,			SHC_FILE_SAVE_AS,	text_edit_menu_callback,			4,	NULL			},
	{ MNU_FILE_QUIT,			SHC_FILE_QUIT,		text_edit_menu_callback,			6,	NULL			},
	{ MNU_OPTIONS,				NULL,				NULL,								0,	"<Branch>"		},
	{ MNU_OPTIONS_WRAP,			SHC_OPTIONS_WRAP,	text_edit_menu_callback,			9,	"<CheckItem>"	},
	{ MNU_HELP,					NULL,				NULL,								0,	"<LastBranch>"	},
	{ MNU_HELP_ABOUT,			NULL,				text_edit_menu_callback,			11,	NULL			},
	{ NULL,						NULL,				NULL,								0,	NULL,			},
	/* Set last item to NULL */
};
#endif

char * text_file_selector(GtkWidget *window, char*title)
{
	char *ptr=NULL;

	DBG_FUNCTION_NAME;

	gtk_window_set_modal (GTK_WINDOW(window), FALSE);
	ptr=file_selector(title, SHOW_FILE_OP_BUTTONS);
	gtk_window_set_modal (GTK_WINDOW(window), TRUE);
	/* This must be freed */
	return ptr;
}

int toggle_wrap_text(GtkWidget *text, int wrap)
{
	DBG_FUNCTION_NAME;

	if(wrap==TRUE)
		SET2(wrap,FALSE);
	else
	 	SET2(wrap,TRUE);
	#ifdef USE_GTK_2_0
	/*
	GTK_WRAP_NONE,
	GTK_WRAP_CHAR,
	GTK_WRAP_WORD
	*/
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(text),(wrap==FALSE)?GTK_WRAP_NONE:GTK_WRAP_WORD);
	#else
	/* TRUE or FALSE */
	gtk_text_set_line_wrap(GTK_TEXT(text), wrap);
	gtk_text_set_word_wrap(GTK_TEXT(text), wrap);
	#endif
	return wrap;
}

int want_to_save(GtkWidget * edit_window,char *path, int *save_as, int *save)
{
	DBG_FUNCTION_NAME;

	/* ask if we want to save or to save as */
	gtk_window_set_modal (GTK_WINDOW(edit_window), FALSE);
	if(dialog_window(STR_SAVE_FILE, STR_FILE_MODIFIED_WANT_TO_SAVE, TWO_BUTTONS, MODAL, NULL)==0)
	{
		if(path==NULL)
			SET(*save_as);
		else
			SET(*save);
		gtk_window_set_modal (GTK_WINDOW(edit_window), TRUE);
		return 0;
	}
	return 1;
}

void clear_the_buffer(GtkWidget *text, int *text_modified)
{
	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		start,
					end;

	DBG_FUNCTION_NAME;
	buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
	#else

	DBG_FUNCTION_NAME;
	gtk_text_backward_delete(GTK_TEXT(text) ,gtk_text_get_length(GTK_TEXT(text)));
	#endif
	RESET(*text_modified);
}

int overwrite(GtkWidget * edit_window, int *flag_A, int *flag_B, char *msg_str)
{
	DBG_FUNCTION_NAME;

	gtk_window_set_modal (GTK_WINDOW(edit_window), FALSE);
	if(	dialog_window(STR_WARNING, msg_str, TWO_BUTTONS, MODAL, NULL)==1)
	{
		RESET(*flag_A);
		RESET(*flag_B);
		return 1;
	}
	gtk_window_set_modal (GTK_WINDOW(edit_window), TRUE);
	return 0;
}

void set_permissions(GtkWidget * edit_window, char *path, mode_t mask)
{
	DBG_FUNCTION_NAME;

	if (chmod(path, mask) == -1)
	{
		gtk_window_set_modal (GTK_WINDOW(edit_window), FALSE);
		error_window(STR_ERROR, STR_CANNOT_SET_FILE_PERMISSIONS_ON_S_S, path);
		gtk_window_set_modal (GTK_WINDOW(edit_window), TRUE);
	}
}

void textedit_window_create(char * title, int x, int y, int border, char *filename)
{
	GtkWidget	*edit_window,
				*table,
				*text,
				*menubar,
				*statusbar_label,
				*statusbar;
#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		iter;
#else
	GdkFont		*fixed_font;
#endif
	unsigned int action=0;
	unsigned int *ptr_action_num;
	int new=0;
	int do_open=0;
	int save=0;
	int save_as=0;
	int wrap=FALSE;
	int quit=0;
	int text_modified=0;
	int *ptr_text_modified;
	char *path=NULL;
	char *temp_ptr;
	mode_t mask;
	FILE *file=NULL;
	struct stat destfileinfo;
	struct stat sourcefileinfo;

	DBG_FUNCTION_NAME;

	sourcefileinfo.st_mtime=0;
	sourcefileinfo.st_mode=0;
	ptr_text_modified = &text_modified;
	ptr_action_num = &action;

	/* Make our own copy of filname so we can free it later */
	if(filename!=NULL)
		path=xstrdup(filename);

	#ifdef DEBUG
	err_msg("textedit_window()");
	#endif

	edit_window = window_create(( x <= 0 ) ? 530 : x, ( y <= 0 ) ? 450 : y, border, title, gtk_main_quit, NULL,  MODAL);
	table = table_create( 20, 22,GTK_WIDGET(edit_window), 0, NOT_HOMOGENEUS);
	menubar = main_menu_create(edit_window, (GtkItemFactoryEntry *) &text_edit_menu_items, ptr_action_num);

#ifdef USE_GTK_2_0
	gtk_table_attach_defaults (GTK_TABLE(table),GTK_WIDGET(menubar), 0, 20, 0, 1);
	text = textpad_create_on_table(EDITABLE, GTK_SIGNAL_FUNC(text_changed_callback), ptr_text_modified, NO_WRAP, SCROLL,
									table,  0, 20, 1,20);
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	statusbar = statusbar_create_on_table(table, &statusbar_label,NULL, 0, 20, 21, 22 );
#else
	fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-130-*-*-*-*-*-*");
	gtk_table_attach_defaults (GTK_TABLE(table),GTK_WIDGET(menubar), 0, 20, 0, 2);
	text = textpad_create_on_table(EDITABLE, GTK_SIGNAL_FUNC(text_changed_callback), ptr_text_modified, NO_WRAP, SCROLL,
									table,  0, 20, 2,21);
	statusbar = statusbar_create_on_table(table, &statusbar_label,NULL, 0, 20, 21, 22 );
#endif


DO_OPEN:

	#ifdef USE_GTK_2_0
	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	#endif

	if(path!=NULL && stat(path, &sourcefileinfo)==0 && (file=open_file(path))!=NULL)
	{
		#ifndef USE_GTK_2_0
		gtk_text_freeze(GTK_TEXT(text));
		#endif
		while((temp_ptr=xfgetc_nbuf(file, DEF_BUFFER_SIZE))!=NULL && GTK_IS_WIDGET(text) )
		{
			#ifdef USE_GTK_2_0
			if(read_and_convert(temp_ptr, buffer, iter, edit_window, MODAL)==1)
				break;
			#else
			gtk_text_insert(GTK_TEXT(text), fixed_font, &text->style->black, NULL, temp_ptr, -1);
			#endif
			while (gtk_events_pending ())
				gtk_main_iteration();
			free(temp_ptr);
		}
		fclose(file);
		#ifndef USE_GTK_2_0
		gtk_text_thaw(GTK_TEXT(text));
		#endif
		gtk_window_set_title(GTK_WINDOW(edit_window), path);
	}
	else
	{
		/*create an empty edit window titled or untitled */
		gtk_window_set_title(GTK_WINDOW(edit_window),(path!=NULL) ? path : STR_UNTITLED );
	}
	RESET(text_modified);

	gtk_widget_show_all(GTK_WIDGET(edit_window));

CONTINUE_OP:
	RESET(action);
	gtk_main();

	switch(action)
	{
		case 1:
			SET(new);
			break;
		case 2:
			SET(new);
			SET(do_open);
			break;
		case 3:
			SET(save);
			break;
		case 4:
			SET(save_as);
			break;
		case 6:
			SET(quit);
			break;
		case 9:
			SET2(wrap,toggle_wrap_text(text, wrap));
			goto CONTINUE_OP;
		case 11:
			about_callback();
			goto CONTINUE_OP;
		default:
			/* this is for the delete_event callback of edit_window */
			SET(quit);
	}

	RESET(action);

	if(IS_NOT_SET(save) && IS_NOT_SET(save_as))
	{
		switch(text_modified)
		{
			case TEXT_MODIFIED:
					if(want_to_save(edit_window, path, &save_as, &save)==0)
						break;
					RESET(text_modified);
			/* FAllTROUGH */
			case TEXT_NOT_MODIFIED:
				if(IS_SET(new))
				{
DO_NEW:
					RESET(new);
					RESET(sourcefileinfo.st_mtime);
					RESET(sourcefileinfo.st_mode);
					gtk_window_set_title(GTK_WINDOW(edit_window), STR_UNTITLED);
					clear_the_buffer(text, &text_modified);
					if(IS_SET(do_open))
					{
						RESET(do_open);
						XFREE(path);
						if((path=text_file_selector(GTK_WIDGET(edit_window), STR_OPEN":"))!=NULL)
							goto DO_OPEN;
					}
					goto CONTINUE_OP;
				}
				goto QUIT;
			/*esac */
		}
	}

	if( IS_SET(save) || IS_SET(save_as))
	{
		if(path==NULL && IS_SET(save)) /* no filename */
		{
			RESET(save);
			if(IS_NOT_SET(text_modified)) /* text not modified, ignoring save.*/
				goto CONTINUE_OP;
			/* else text was modified  doing save_as instead */
			SET(save_as);
		}
		if(IS_SET(text_modified))
		{
			if(IS_SET(save_as))
			{
DO_SAVE_AS:
				XFREE(path);
				if((path=text_file_selector(GTK_WIDGET(edit_window), STR_SAVE_AS":"))==NULL)
				{
					RESET(save_as);
					goto CONTINUE_OP;
				}
			}
			if(stat(path, &destfileinfo)==0)
			{
				/* File exists Overwrite ? */
				if(overwrite(edit_window, &save, &save_as, STR_DEST_FILE_EXIST)==1)
					goto CONTINUE_OP;

				mask = destfileinfo.st_mode;

				if(sourcefileinfo.st_mtime!=0 && sourcefileinfo.st_mtime > destfileinfo.st_mtime )
				{
					if(overwrite(edit_window, &save, &save_as, STR_DEST_FILE_NEWER)==1)
						goto CONTINUE_OP;
				}
			}
			else
			{
				if(sourcefileinfo.st_mode!=0)
					mask=sourcefileinfo.st_mode;
				else
					mask = 0644; /* Use default file permissions */
			}
			save_file(path, text);
			set_permissions(edit_window, path, mask);
			gtk_window_set_title (GTK_WINDOW(edit_window), path);
		}
		else
		{
			/* Want to create a new file with name? */
			if(IS_SET(save_as))
				goto DO_SAVE_AS;
			if(IS_SET(new))
				goto DO_NEW;
		}
		RESET(save);
		RESET(save_as);
		RESET(text_modified);
		if(IS_SET(new))
			goto DO_NEW;
		if(IS_SET(quit))
			goto QUIT;
		goto CONTINUE_OP;
	}
QUIT:
	XFREE(path);
	/* Why GTK_IS_WIDGET ? */
	if(GTK_IS_WIDGET(edit_window))
		gtk_widget_destroy(edit_window);
	return;
}

/***************End of Text file edit Widget ********************************/
