/*
*
* Gtk+1.2-Gtk+2.0  Utility routines
* Copyright (C) 2002-2003-2004 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

/********************* Text file viewer Widget *******************************/

#include <gtk/gtk.h>
#include <stdlib.h>

void textview_window_close(GtkWidget *widget, GtkWidget *window)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	gtk_widget_destroy(window);
}

#ifdef USE_GTK_2_0
void wrap_callback(GtkWidget *widget, GtkWidget *text)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	if(gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(text)) == GTK_WRAP_NONE)
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
	else
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_NONE);
}
#else
void wrap_callback(GtkWidget *widget, GtkText *text)
{
	DBG_FUNCTION_NAME;
	exit_if_null(widget);

	gtk_text_set_line_wrap ( GTK_TEXT (text), (text->line_wrap == FALSE) ? TRUE : FALSE);
	gtk_text_set_word_wrap ( GTK_TEXT (text), (text->word_wrap == FALSE) ? TRUE : FALSE);
}
#endif

void textview_window_create(char * title,int x, int y, int border, int modal_flag, FILE *fd, char *ptr)
{
	GtkWidget		*edit_window,
					*table,
					*text,
					*bbox,
					*check_button,
					*button;
	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		iter;
	#else
	GdkFont		*fixed_font;
	#endif
	char *temp_ptr;

	DBG_FUNCTION_NAME;

	/* Make the gui first  */
	edit_window = window_create((x<=0)?530:x, (y<=0)?430:y, border, title,GTK_SIGNAL_FUNC(textview_window_close) ,
								&edit_window ,modal_flag);
	table = table_create( 20, 18, edit_window , 0, NOT_HOMOGENEUS);
	text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, table,  0, 20, 0, 16);
	check_button = check_create_on_table(STR_WRAP, GTK_SIGNAL_FUNC(wrap_callback), text, INACTIVE, table, 0, 3, 16, 17);
	bbox = default_button_box_create_on_table( HORIZONTAL, table, 3, 20, 16, 18);
	button = button_create_in_container(STR_OK, GTK_SIGNAL_FUNC(textview_window_close) , edit_window ,bbox);
	gtk_widget_show_all(edit_window);

	#ifdef USE_GTK_2_0
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	#else
	/* Load a fixed font */
	fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-130-*-*-*-*-*-*");
	#endif

	if(ptr!=NULL)
	{
		/* limit this to DEF_BUF_SIZE */
		temp_ptr=(char *)xmalloc(DEF_BUFFER_SIZE);
		snprintf(temp_ptr,DEF_BUFFER_SIZE,"%s",ptr);
		#ifdef USE_GTK_2_0
		read_and_convert(temp_ptr, buffer, iter, edit_window, modal_flag);
		#else
		gtk_text_insert(GTK_TEXT(text), fixed_font, &text->style->black, NULL, temp_ptr, -1);
		#endif
		free(temp_ptr);
	}
	else
	{

		/* Read from fd */
		if(fd==NULL)
		{
			text_view_error(edit_window, modal_flag ,  STR_CANNOT_OPEN_FILE);
		}
		else
		{
			#ifndef USE_GTK_2_0
			gtk_text_freeze(GTK_TEXT(text));
			#endif
			while((temp_ptr=xfgetc_nbuf(fd, DEF_BUFFER_SIZE))!=NULL && GTK_IS_WIDGET(text) )
			{
				#ifdef USE_GTK_2_0
				if(read_and_convert(temp_ptr, buffer, iter, edit_window, modal_flag)==1)
					break;
				#else
				gtk_text_insert(GTK_TEXT(text), fixed_font, &text->style->black, NULL, temp_ptr, -1);
				#endif
				while (gtk_events_pending ())
					gtk_main_iteration();
				free(temp_ptr);
			}
			#ifndef USE_GTK_2_0
			gtk_text_thaw(GTK_TEXT(text));
			#endif
		}
	}
}

/***************End of Text file viewer Widget ********************************/
