/*
*
* Gtk+1.2-Gtk+2.2 Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>

#ifdef L_exit_if_null
void exit_if_null(void * test_arg)
{
	/* Mostly used to quiet compiler warnings  or for debugging */
	if(test_arg==NULL)
	{
		#ifdef DEBUG
		err_msg("test_arg is NULL");
		#endif
		exit(EXIT_FAILURE);
	}
}
#endif
#ifdef L_do_nothing
/* gtk-2.0 :ok */
void do_nothing(GtkWidget *widget)
{
	/* To quiet compiler warnings */
	exit_if_null(widget);
	return;
}
#endif

/********************* Text box widget ***************************************/

/* gtk-2.0 :ok */
#ifdef L_text_box_create_on_table
GtkWidget *text_box_create_on_table(char *text, int editable, GtkWidget *table,
								 int start_col, int end_col, int start_row, int end_row)
{
	GtkWidget *entry_box;

	entry_box = gtk_entry_new();
	gtk_table_attach_defaults (GTK_TABLE(table),entry_box, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	if(text!=NULL)
		gtk_entry_set_text(GTK_ENTRY(entry_box),text);
	#ifdef USE_GTK_2_0
	gtk_editable_set_editable(GTK_EDITABLE (entry_box),(editable==EDITABLE)?TRUE:FALSE);
	#else
	gtk_entry_set_editable (GTK_ENTRY (entry_box),(editable==EDITABLE)?TRUE:FALSE);
	#endif
	return entry_box;
}
#endif
/********************* End of Text box widget ********************************/

/********************* Standard radio button widget **************************/

/* gtk-2.0 :ok */
#ifdef L_get_radio_button_group
GSList* get_radio_button_group(GtkRadioButton *radio_button)
{

	#ifdef USE_GTK_2_0
	return (gtk_radio_button_get_group(GTK_RADIO_BUTTON (radio_button)));
	#else
	return (gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button)));
	#endif
}
#endif
/* gtk-2.0 :ok */
#ifdef L_radio_create
GtkWidget *radio_create( GtkWidget *last_of_group , char *label,
			 GtkSignalFunc func, gpointer func_data, int active )
{
	GtkWidget *button;

	if( last_of_group==NULL)
	{
		button = gtk_radio_button_new_with_label(NULL, label);
	}
	else
		#ifdef USE_GTK_2_0
		button = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON (last_of_group)), label);
		#else
		button = gtk_radio_button_new_with_label(gtk_radio_button_group (GTK_RADIO_BUTTON (last_of_group)) , label);
		#endif
 	if(func!=NULL)
	{
		#ifdef USE_GTK_2_0
		g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC(func), func_data);
		#else
		gtk_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC(func), func_data);
		#endif
	}
	#ifdef DEBUG
	else
		err_msg("radio_create(): GtkSignalFunc func == NULL");
	#endif

	if(active==ACTIVE)
	{
		/* gtk_toggle_button_set_active is not needed on first button (last_of_group==NULL)
			because it is set active by default, but to be sure.......
		*/
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),TRUE);
		gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (button));
	}
	return button;
}
#endif
/****************End of Standard radio button widget *************************/

/********************* Radio button on table widget **************************/
/* gtk-2.0 :ok */
#ifdef L_radio_create_on_table
GtkWidget *radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func, gpointer func_data,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row )
{
	GtkWidget *button;
	button = radio_create(last_of_group, label, func, func_data , active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
#endif
/* gtk-2.0 :ok */
#ifdef L_default_radio_create_on_table
GtkWidget *default_radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row )
{
	GtkWidget *button;
	button = radio_create(last_of_group, label, func, label ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
#endif
/****************End of radio button on table widget *************************/

/********************* Check button widgets **************************/
/* gtk-2.0 :ok */
#ifdef L_check_create
GtkWidget *check_create(char *label, GtkSignalFunc func, gpointer func_data, int active )
{
	GtkWidget *button;

	button = gtk_check_button_new_with_label(label);
	if(func!=NULL)
	{
		#ifdef USE_GTK_2_0
		g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC(func), func_data);
		#else
		gtk_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC(func), func_data);
		#endif
	}
	#ifdef DEBUG
	else
		err_msg("check_create(): GtkSignalFunc func == NULL");
	#endif

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),(active==ACTIVE)?TRUE:FALSE);

	return button;
}
#endif
/* gtk-2.0 :ok */
#ifdef L_check_create_on_table
GtkWidget *check_create_on_table(char *label, GtkSignalFunc func, gpointer func_data, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = check_create(label, func, func_data ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
#endif
/* gtk-2.0 :ok */
#ifdef L_default_check_create_on_table
GtkWidget *default_check_create_on_table(char *label, GtkSignalFunc func, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = check_create(label, func, label ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
#endif
/***************End of Check button widgets **********************************/

/********************* Standard button widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_button_create
GtkWidget *button_create(char *label, GtkSignalFunc func, gpointer func_data )
{
	GtkWidget *button;

	button = gtk_button_new_with_label (label);
	if(func!=NULL)
	{
		#ifdef USE_GTK_2_0
		g_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (func), func_data);
		#else
		gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (func), func_data);
		#endif
	}
	#ifdef DEBUG
	else
		err_msg("button_create(): GtkSignalFunc func == NULL");
	#endif
	return button;
}
#endif
/****************End of Standard button widget *******************************/

/********************* Button on table widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_button_create_on_table
GtkWidget *button_create_on_table(char *label, GtkSignalFunc func, gpointer func_data,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;

	button = button_create(label, func, func_data );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	/*gtk_table_attach(GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row),
					(GtkAttachOptions) ( GTK_EXPAND | GTK_SHRINK), (GtkAttachOptions) (GTK_SHRINK), 0, 0);*/
	return button;
}
#endif
/****************End of button on table widget ******************************/

/********************* Button in container widget ***************************/
/* gtk-2.0 :ok */
#ifdef L_button_create_in_container
GtkWidget *button_create_in_container(char *label, GtkSignalFunc func, gpointer func_data, GtkWidget *container)
{
	GtkWidget *button;

	button = button_create(label, func, func_data);
	gtk_container_add (GTK_CONTAINER (container), button);
	return button;
}
#endif
/*************End of Button in container widget *****************************/

/********************* Buttonbox on table widget ****************************/
/* gtk-2.0 :ok */
#ifdef L_button_box_create_on_table
GtkWidget *button_box_create_on_table(	int orientation, int layout,  int border,
										int spacing, int child_size_x, int child_size_y, GtkWidget *table,
										int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *bbox;

	if(orientation != HORIZONTAL && orientation != VERTICAL)
	{
		orientation=HORIZONTAL;
		#ifdef DEBUG
		err_msg("button_box_create_on_table(): orientation != HORIZONTAL or VERTICAL use default HORIZONTAL");
		#endif
	}

	if(orientation==HORIZONTAL)
		bbox = gtk_hbutton_box_new ();
	else
		bbox = gtk_vbutton_box_new ();


	gtk_container_set_border_width (GTK_CONTAINER (bbox),abs(border));
	gtk_table_attach_defaults (GTK_TABLE(table), bbox, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), layout );
	#ifdef USE_GTK_2_0
	/* Do we need some substitute for gtk_button_box_set_child_size ?*/
	gtk_box_set_spacing (GTK_BOX (bbox), abs(spacing));
	/* To silence a compiler warning about unused vars */
	child_size_x=child_size_y=0;
	#else
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox),abs(spacing));
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (bbox),abs(child_size_x),abs(child_size_y));
	#endif
	return bbox;
}
#endif
/* gtk-2.0 :ok */
#ifdef L_default_button_box_create_on_table
GtkWidget *default_button_box_create_on_table(	int orientation, GtkWidget *table,
										 		int start_col, int end_col,
												int start_row, int end_row )
{
	GtkWidget *bbox;

	/* These default values look good to me. If you don't like it
		use button_box_create_on_table()	to set your own */
	bbox = button_box_create_on_table( orientation,  GTK_BUTTONBOX_END, 10, 20, 85, 30,
								 		table, start_col, end_col, start_row, end_row );
	return bbox;
}
#endif
/***************End of Buttonbox on table widget ****************************/

/********************* Label on table widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_label_create_on_table
GtkWidget *label_create_on_table(char *label_text,GtkWidget *table,
				int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *label;
	label=gtk_label_new(label_text);
	gtk_table_attach_defaults (GTK_TABLE(table), label, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return label;
}
#endif
/***************End of Label on table widget ********************************/

/********************* Standard table widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_table_create
GtkWidget *table_create(int cols , int rows,  GtkWidget* window , int border, int homogeneus)
{
	GtkWidget *table;

	if(rows <= 0) rows=1;
	if(cols <= 0) cols=1;

	#ifdef DEBUG
	if(window==NULL)
	{
		err_msg("table_create(): GtkWidget *window == NULL");
		exit(EXIT_FAILURE);
	}
	#endif

	table = gtk_table_new (rows, cols, TRUE);

	#ifdef USE_GTK_2_0
	/* THIS MAKES THINGS LOOK BETTER */
	gtk_table_set_homogeneous(GTK_TABLE(table), homogeneus);
	#else
		#ifdef DEBUG
		if(homogeneus!= -1)
			err_msg("table_create(): HOMOGENEUS/NOT_HOMOGENEUS flag not supported by Gtk+1.x.x libs");
		#endif
		/* to shut up a compiler warning */
		homogeneus=IGNORED;
	#endif

	gtk_container_add (GTK_CONTAINER (window), table);
	gtk_container_set_border_width (GTK_CONTAINER (table),abs(border) );
	return table;
}
#endif

/********************* Standard table widget ********************************/

/********************* Standard window widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_window_create
GtkWidget *window_create( int x, int y, int border, const char *title, GtkSignalFunc func,  void *func_data, int modal)
{
		GtkWidget	*window;
		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		if(func!=NULL)
		{
			#ifdef USE_GTK_2_0
			g_signal_connect (window, "delete_event", GTK_SIGNAL_FUNC (func), func_data);
			#else
			gtk_signal_connect (GTK_OBJECT (window), "delete_event", GTK_SIGNAL_FUNC (func), func_data);
			#endif
		}
		#ifdef DEBUG
		else
			err_msg("window_create(): GtkSignalFunc func == NULL");
		#endif

		if(x==0)x=640;
		if(y==0)y=480;

		#ifdef USE_GTK_2_0
		gtk_window_set_default_size(GTK_WINDOW(window),abs(x),abs(y));
		#else
		gtk_widget_set_usize(GTK_WIDGET(window),abs(x),abs(y));
		#endif
		gtk_window_set_title(GTK_WINDOW(window), title);
		gtk_container_set_border_width (GTK_CONTAINER (window), abs(border));
		gtk_window_set_modal(GTK_WINDOW(window),(modal==MODAL) ? TRUE : FALSE);
		return window;
}
#endif

/********************* Standard window widget ********************************/

/********************* Frame on table widget ********************************/
/* gtk-2.0 :ok */
#ifdef L_frame_create_on_table
GtkWidget *frame_create_on_table(char *label, GtkWidget *table,
											 int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget *frame;

	frame = gtk_frame_new(label);
	gtk_table_attach_defaults(GTK_TABLE(table), frame,  abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return frame;
}
#endif
/**************End of Frame on table widget ********************************/

/************** Create a separator *****************************************/
/* gtk-2.0 :ok */
#ifdef L_separator_create_on_table
GtkWidget *separator_create_on_table(int orientation, GtkWidget *table, int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget *separator;

	if(orientation != HORIZONTAL && orientation != VERTICAL)
	{
		orientation=HORIZONTAL;
		#ifdef DEBUG
		err_msg("separator_create_on_table(): orienation != HORIZONTAL or VERTICAL use default HORIZONTAL");
		#endif
	}

	if(orientation==HORIZONTAL)
		separator=gtk_hseparator_new();
	else
		separator=gtk_vseparator_new();

	gtk_table_attach_defaults(GTK_TABLE(table), separator ,abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return separator;
}
#endif
/************** End of Create a separator **********************************/

/************** Create a Multi column scrolled list ************************/
#ifdef L_string_list_create_on_table
#ifdef USE_GTK_2_0
GtkWidget      *string_list_create_on_table(int num,	GtkSignalFunc func,
											GtkWidget *table, int start_col, int end_col,
											int start_row, int end_row ,int show_hide, ...)
{
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GtkListStore		*store;
	GtkWidget			*scrolled_window;
	GtkWidget			*tree;
	GtkTreeSelection *select;
 	GType *types;
	va_list titles;
	int i;

	/* This is the scrolled window to put the cList widget inside */
	scrolled_window=gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);

	types = (GType *) malloc(num*sizeof(GType));
	for( i=0; i< num ; i++)
		types[i]=G_TYPE_STRING;

	store = gtk_list_store_newv (num, types);
	free(types);
	tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(store));

	/* Setup the selection handler */
	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);

	if(func!=NULL)
		g_signal_connect (select, "changed", G_CALLBACK (func), NULL);
	#ifdef DEBUG
	else
		err_msg("string_list_create_on_table(): func == NULL");
	#endif

	gtk_container_add(GTK_CONTAINER(scrolled_window),GTK_WIDGET(tree));
	gtk_table_attach_defaults(	GTK_TABLE(table),GTK_WIDGET(scrolled_window),
							 	abs(start_col),		abs(end_col),
							 	abs(start_row),		abs(end_row));
	/* The view now holds a reference.  We can get rid of our own reference */
	g_object_unref (G_OBJECT (store));

	va_start(titles, show_hide);
	for(i=0; i<num ;i++)
	{
		/* Create a cell renderer */
		renderer = gtk_cell_renderer_text_new ();
		/* Create a column, associating the "text" attribute of the  cell_renderer to the first column of the model */

		column = gtk_tree_view_column_new_with_attributes ((show_hide == SHOW) ? va_arg(titles, char *) : "" ,
								renderer,
								"text",
								0,
								NULL);
		/* Add the column to the view. */
		gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
	}
	va_end(titles);
	gtk_tree_view_columns_autosize  (GTK_TREE_VIEW(tree));
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_ETCHED_IN);
	return tree;
}
#else
GtkWidget      *string_list_create_on_table(int num,	GtkSignalFunc func,
											GtkWidget *table, int start_col, int end_col,
											int start_row, int end_row ,int show_hide, ...)
{

	GtkWidget      *list;
	GtkWidget      *scrolled_window;
	va_list titles;

	/* This is the scrolled window to put the cList widget inside */
	scrolled_window=gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);

	gtk_table_attach_defaults(	GTK_TABLE(table),	scrolled_window,
							 	abs(start_col),		abs(end_col),
							 	abs(start_row),		abs(end_row));


	va_start(titles, show_hide);
	list = gtk_clist_new_with_titles( abs(num),(char **)titles);
	va_end(titles);

	if(func!=NULL)
		gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(func), (gpointer) list);
	#ifdef DEBUG
	else
		err_msg("string_list_create_on_table(): func == NULL");
	#endif

	/* It isn't necessary to shadow the border, but it looks nice :) */
    gtk_clist_set_shadow_type (GTK_CLIST(list), GTK_SHADOW_OUT);

	gtk_container_add(GTK_CONTAINER(scrolled_window), list);

	if(show_hide==SHOW)
		gtk_clist_column_titles_show((GtkCList *) list);
	else
		gtk_clist_column_titles_hide((GtkCList *) list);

	return list;
}
#endif /* USE_GTK_2_0 */
#endif
/******** End of Create a Multi column scrolled list ************************/

/************** Create a statusbar **********************************/
#ifdef L_statusbar_create_on_table
GtkWidget * statusbar_create_on_table(GtkWidget *table, GtkWidget **label, char* labeltext, int start_col, int end_col, int start_row, int end_row )
{

	GtkWidget *frame;

	frame=frame_create_on_table(NULL,table, start_col, end_col, start_row, end_row );

	/* Allowed values for shadow type are:
		GTK_SHADOW_NONE,
		GTK_SHADOW_IN,
		GTK_SHADOW_OUT,
		GTK_SHADOW_ETCHED_IN,
		GTK_SHADOW_ETCHED_OUT
	*/

	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_IN);
	*label=gtk_label_new(labeltext);
	gtk_misc_set_alignment((GtkMisc *)*label,  0.5, 0.5);
	gtk_container_add(GTK_CONTAINER(frame),GTK_WIDGET(*label));
	return frame;
}
#endif
/************** End of Create a statusbar **********************************/

/******************* Create a Textpad **************************************/
#ifdef L_textpad_create_on_table

#include <stdlib.h>

#ifdef USE_GTK_2_0
GtkWidget *textpad_create_on_table(int editable, GtkSignalFunc func, void * func_arg, int wrap, int scroll, GtkWidget *table,
										  int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget		*text,
					*frame,
					*sw;

	text = gtk_text_view_new ();
	gtk_text_buffer_create_tag (gtk_text_view_get_buffer (GTK_TEXT_VIEW (text)), "monospace", "family", "monospace", NULL);
	frame = gtk_frame_new(NULL);
	gtk_table_attach_defaults(GTK_TABLE(table), frame, abs(start_col), abs(end_col), abs(start_row),abs(end_row));
	gtk_text_view_set_editable (GTK_TEXT_VIEW(text), FALSE);
	if(editable==EDITABLE)
	{
		gtk_text_view_set_editable (GTK_TEXT_VIEW(text), TRUE);
		/* It's best to connect a signal here to have Gtk tell us if the text gets modified! */
		/* but only for text editing widgets so also NULL is allowed */
		if(func!=NULL)
		{
			g_signal_connect(gtk_text_view_get_buffer (GTK_TEXT_VIEW (text)),"changed", GTK_SIGNAL_FUNC(func),func_arg);
		}
		#ifdef DEBUG
		else
			err_msg("text_create_on_table():EDITABLE but func == NULL");
		#endif
	}

	if(wrap==WRAP)
	{
		gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(text),GTK_WRAP_WORD);
	}

	if(scroll==SCROLL)
	{
		sw = gtk_scrolled_window_new (NULL, NULL);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
						GTK_POLICY_AUTOMATIC,
						GTK_POLICY_ALWAYS);
		gtk_container_add (GTK_CONTAINER (frame), sw);
		gtk_container_add (GTK_CONTAINER (sw), text);
	}
	else
		gtk_container_add (GTK_CONTAINER (frame), text);
	gtk_widget_realize(text);
	return text;
}

#else

GtkWidget *textpad_create_on_table(int editable, GtkSignalFunc func, void * func_arg, int wrap, int scroll, GtkWidget *table,
										  int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget		*new_text,
					*vscrollbar;

	new_text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable (GTK_TEXT (new_text), FALSE);
	if(editable==EDITABLE)
	{
		gtk_text_set_editable (GTK_TEXT (new_text), TRUE);
		/* It's best to connect a signal here to have Gtk tell us if the text gets modified! */
		/* but only for text editing widgets so also NULL is allowed */
		if(func!=NULL)
			gtk_signal_connect (GTK_OBJECT (new_text),"changed", GTK_SIGNAL_FUNC(func),func_arg);
		#ifdef DEBUG
		else
			err_msg("text_create_on_table():EDITABLE but GtkSignalFunc func == NULL");
		#endif
	}

	gtk_text_set_line_wrap(GTK_TEXT(new_text),(wrap==WRAP)?TRUE:FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(new_text),(wrap==WRAP)?TRUE:FALSE);

	gtk_table_attach_defaults(GTK_TABLE(table), new_text, abs(start_col), abs(end_col), abs(start_row),abs(end_row));
	if(scroll==SCROLL)
	{
		vscrollbar = gtk_vscrollbar_new(GTK_TEXT(new_text)->vadj);
		gtk_table_attach(GTK_TABLE(table), vscrollbar,	abs(end_col), abs(end_col)+1,
																		abs(start_row),abs(end_row),
																		GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	}
	gtk_widget_realize(new_text);
	return new_text;
}

#endif /* USE_GTK_2_0 */
#endif /*L_textpad_create_on_table*/


/************ End of Create a Textpad **************************************/

/************ Create a Main Menubar   **************************************/

GtkWidget * main_menu_create( GtkWidget *window, GtkItemFactoryEntry *menu_items, gpointer ptr_action_num )
{
	GtkWidget *menubar;
	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	int n;

	accel_group = gtk_accel_group_new ();

	/* This function initializes the item factory.
		Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
				or GTK_TYPE_OPTION_MENU.
		Param 2: The path of the menu.
		Param 3: A pointer to a gtk_accel_group.  The item factory sets up
				the accelerator table while generating menus.
	*/
	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);

	#ifdef USE_GTK_2_0
	g_object_set_data_full (G_OBJECT (window), "<main>", item_factory, (GDestroyNotify) g_object_unref);
	#endif

	for(n=0;;n++)
	{
		if(menu_items[n].path==NULL)
			break;
		/*	1 in arg 4 defines the type of callback
			(*GtkItemFactoryCallback1) (gpointer callback_data, guint callback_action, GtkWidget *widget);
			arg 3 is gpointer callback_data;
			guint callback_action  is field 4 of  GtkItemFactoryEntry text_edit_menu_items */
		gtk_item_factory_create_item(item_factory, &menu_items[n], ptr_action_num, 1);
	}

	/* Attach the new accelerator group to the window. */

	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

	/* Finally, return the actual menu bar created by the item factory. */
	menubar = gtk_item_factory_get_widget (item_factory, "<main>");
	if (menubar)
		return menubar;
	else
	{
		#ifdef DEBUG
		err_msg("menubar==NULL");
		#endif
		exit(EXIT_FAILURE);
	}
}

/************ End of Create a Main Menubar**********************************/


