/*
*
* Gtk+1.2-Gtk+2.2 Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/
#ifndef	__HAVE_MYGTK_
#define	__HAVE_MYGTK_ 1

#include <gtk/gtk.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

/* This is to avoid a compiler warning for `index` with -Wshadow and Gtk-+-1.2 libs */
#ifndef USE_GTK_2_0
	#ifdef IS_LINUX
		#undef  __USE_BSD
		#include <string.h>
		#define  __USE_BSD
	#endif
#else
	#include <string.h>
#endif

#define DEF_BUFFER_SIZE		 512
#define NOT_EDITABLE	((int) 0)
#define EDITABLE		((int) 1)
#define ACTIVE 			((int) 0)
#define INACTIVE		((int) 1)
#define HORIZONTAL		((int) 0)
#define VERTICAL		((int) 1)
#define	NOT_MODAL		((int) 0)
#define	MODAL			((int) 1)
#define SHOW			((int) 0)
#define HIDE			((int) 1)
#define WRAP			((int) 0)
#define NO_WRAP			((int) 1)
#define SCROLL			((int) 0)
#define NO_SCROLL		((int) 1)

#define TEXT_NOT_MODIFIED	((int) 0)
#define TEXT_MODIFIED		((int) 1)

#define	HIDE_FILE_OP_BUTTONS	((int) 0)
#define	SHOW_FILE_OP_BUTTONS	((int) 1)

#define STRERROR_YES		((int) 1)
#define STRERROR_NO			((int) 0)

/*#define	ONE_BUTTON		((int) 1)
#define	TWO_BUTTONS		((int) 2)
#define	THREE_BUTTONS	((int) 3)*/

#define	ONE_BUTTON				((int) 1)
#define	ONE_BUTTON_ONE_CHECK	((int) 2)
#define	TWO_BUTTONS				((int) 3)
#define	TWO_BUTTONS_ONE_CHECK	((int) 4)
#define	THREE_BUTTONS			((int) 5)

#define	NOT_HOMOGENEUS	((int) 0)
#define HOMOGENEUS		((int) 1)
#define IGNORED			((int) -1)

#define	NOT_FIRST_TIME	((int) 0)
#define FIRST_TIME		((int) 1)

#ifdef DEBUG
#define RESET(x)		err_msg(#x": %d",(x)=0)
#define SET(x) 			err_msg(#x": %d",(x)=1)
#define SET2(x,y) 		err_msg(#x": %d",(x)=(y))
#else
#define RESET(x) 		(x)=0
#define SET(x) 			(x)=1
#define SET2(x,y)		(x)=(y)
#endif
#define IS_SET(x)		(x)==1
#define IS_NOT_SET(x)	(x)==0

void do_nothing(GtkWidget *widget);
void exit_if_null(void * test_arg);

/********************* Text box widget ***************************************/

GtkWidget *text_box_create_on_table(char *text, int editable, GtkWidget *table,
								 int start_col, int end_col, int start_row, int end_row);

/********************* End of Text box widget ********************************/

/********************* Standard radio button widget **************************/

GSList* get_radio_button_group(GtkRadioButton *radio_button);

GtkWidget *radio_create(GtkWidget *last_of_group, char *label,
			 GtkSignalFunc func, gpointer func_data, int active );

/****************End of Standard radio button widget *************************/

/********************* Radio button on table widget **************************/

GtkWidget *radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func, gpointer func_data,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row );
GtkWidget *default_radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row );

/****************End of radio button on table widget *************************/

/********************* Check button widgets **************************/

GtkWidget *check_create(char *label, GtkSignalFunc func, gpointer func_data, int active );
GtkWidget *check_create_on_table(char *label, GtkSignalFunc func, gpointer func_data, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );
GtkWidget *default_check_create_on_table(char *label, GtkSignalFunc func, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );

/***************End of Check button widgets **********************************/

/********************* Standard button widget ********************************/

GtkWidget *button_create(char *label, GtkSignalFunc func, gpointer func_data );

/****************End of Standard button widget *******************************/

/********************* Button on table widget ********************************/

GtkWidget *button_create_on_table(char *label, GtkSignalFunc func, gpointer func_data,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );

/****************End of button on table widget ******************************/

/********************* Button in container widget ***************************/

GtkWidget *button_create_in_container(char *label, GtkSignalFunc func, gpointer func_data,
			GtkWidget *container);

/*************End of Button in container widget *****************************/

/********************* Buttonbox on table widget ****************************/

GtkWidget *button_box_create_on_table(int orientation, int layout,  int border,
										 int spacing, int child_size_x, int child_size_y, GtkWidget *table,
										 int start_col, int end_col, int start_row, int end_row );
GtkWidget *default_button_box_create_on_table(	int orientation, GtkWidget *table,
										 		int start_col, int end_col,
												int start_row, int end_row );

/***************End of Buttonbox on table widget ****************************/

/********************* Label on table widget ********************************/

GtkWidget *label_create_on_table(char *label_text,GtkWidget *table,
				int start_col, int end_col, int start_row, int end_row );

/***************End of Label on table widget ********************************/

/********************* Standard table widget ********************************/

GtkWidget *table_create(int cols, int rows , GtkWidget* window , int border, int homogeneus);

/********************* Standard table widget ********************************/

/********************* Standard window widget ********************************/

GtkWidget *window_create( int x, int y, int border, const char *title, GtkSignalFunc func,
			void *func_data, int modal);

/********************* Standard window widget ********************************/

/********************* Frame on table widget ********************************/

GtkWidget *frame_create_on_table(char *label, GtkWidget *table,
											 int start_col,int end_col, int start_row, int end_row );

/**************End of Frame on table widget ********************************/

/********************* Dialog window widget **********************************/
void dialog_ok_callback(GtkWidget *widget,  int *ptr_dialog_retval);
void dialog_no_callback(GtkWidget *widget,  int *ptr_dialog_retval);
void dialog_third_callback(GtkWidget *widget,  int *ptr_dialog_retval);
int  dialog_window(const char *title, char *labeltext , int type, int modal, char *third_button_label);

/**************End of Dialog window widget **********************************/

/********************* Error window widget **********************************/

void error_window(const char* title, char *message, char* varname);

/***************End of  Error window widget **********************************/

/********************* For the file selection widget *************************/

void file_sel_ok(GtkWidget *widget, int *ptr_ok);
char *file_selector(char *title, int fileop);

/*************End of the file selection widget********************************/

/********************* Text file viewer Widget *******************************/

GtkWidget *textpad_create_on_table(	int editable, GtkSignalFunc func, void *func_arg,
									int wrap, int scroll, GtkWidget *table,
									int start_col,int end_col, int start_row, int end_row );
void textview_window_close(GtkWidget *widget, GtkWidget *window);
#ifdef USE_GTK_2_0
void wrap_callback(GtkWidget *widget, GtkWidget *text);
#else
void wrap_callback(GtkWidget *widget, GtkText *text);
#endif
void textview_window_create(char * title,int x, int y, int border, int modal_flag, FILE *fd, char *ptr);

/**************End of Text file viewer Widget ********************************/

/************** Create a separator on table   ********************************/

GtkWidget *separator_create_on_table(int direction, GtkWidget *table,
									 int start_col,int end_col, int start_row, int end_row );

/**********End of  Create a separator on table********************************/

/************** Create a Multi column scrolled list ************************/

GtkWidget      *string_list_create_on_table(int num,GtkSignalFunc func,
											GtkWidget *table, int start_col, int end_col,
											int start_row, int end_row ,int show_hide, ...);

/******** End of Create a Multi column scrolled list ************************/

/************** Create a statusbar **********************************/

GtkWidget * statusbar_create_on_table(GtkWidget *table, GtkWidget **label, char* labeltext, int start_col, int end_col, int start_row, int end_row );

/******* End of Create a statusbar **********************************/

/************** Create a Permissions window **********************************/


int (*chown_func)(const char *, uid_t, gid_t);
int chmodAction(const char *fileName, struct stat *statbuf, void* junk);
int chownAction(const char *fileName, struct stat *statbuf, void* junk);
mode_t get_mode (const char *string);
void set_r_mode_callback(GtkToggleButton *widget, char *position);
void set_w_mode_callback(GtkToggleButton *widget, char *position);
void set_x_mode_callback(GtkToggleButton *widget, char *position);
void set_s_mode_callback(GtkToggleButton *widget, char *position);
void set_t_mode_callback(GtkToggleButton *widget, char *position);
void recursive_callback(GtkToggleButton *widget, int *ptr_recursive);
void quit_callback(GtkWidget *widget, int *ptr_execute);
void PermissionWindow(char *filename);

/*****************End of Permissions window **********************************/

/************** Text window Common  functions ********************************/

void text_file_op_error(char *fmt, char *filename, int flag);
FILE *open_file(char *filename );
#ifdef USE_GTK_2_0
int read_and_convert(char * temp_ptr, GtkTextBuffer *buffer, GtkTextIter iter , GtkWidget *window, int modal_flag);
#endif
void text_view_error(GtkWidget *window, int modal_flag , char * msg_string);

/************** Text window Common  functions ********************************/

/************** Create a Text Edit window **********************************/

void save_file(char *filename, GtkWidget *text);
void text_changed_callback(GtkWidget *widget, int *ptr_text_modified);
void about_callback(void);
void text_edit_menu_callback(unsigned int* callback_data, guint callback_action, GtkWidget *widget);
char * text_file_selector(GtkWidget *window, char*title);
int toggle_wrap_text(GtkWidget *text, int wrap);
int want_to_save(GtkWidget * edit_window,char *path, int *save_as, int *save);
void clear_the_buffer(GtkWidget *text, int *text_modified);
int overwrite(GtkWidget * edit_window, int *flag_A, int *flag_B, char *msg_str);
void set_permissions(GtkWidget * edit_window, char *path, mode_t mask);
void textedit_window_create(char * title, int x, int y, int border, char *filename);

/************** End of a Text Edit window **********************************/

/************ Create a Main Menubar   **************************************/

GtkWidget * main_menu_create( GtkWidget *window, GtkItemFactoryEntry *menu_items, gpointer ptr_action_num );

/************ Create a Main Menubar   **************************************/

/************** Create a Text Edit window **********************************/

void about_window_close(GtkWidget *widget, GtkWidget *window);
#ifdef USE_GTK_2_0
void about_wrap_callback(GtkWidget *widget, GtkWidget *text);
#else
void about_wrap_callback(GtkWidget *widget, GtkText *text);
#endif
void license_callback(GtkWidget *widget, int *ptr_status);
int about_window_create(char * title,int x, int y, int border, int modal_flag, char *labeltext, FILE *fd, int first_time);

/************** Create a Text Edit window **********************************/

#endif /* __HAVE_MYGTK_  */
