/*
*
* XFPROT Copyright (C) 2002-2003-2004 by Tito Ragusa <tito-wolit@tiscali.it>
*
* XFPROT is a graphical frontend for the F-Prot Antivirus
* for Linux Small Business Edition. It was based upon the
* XSTEP-3.5.1 library until version 0.19beta.
* Starting from version 0.20beta it is based upon Gtk-1.2.10.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/* This is to avoid a compiler warning for `index` with -Wshadow and Gtk-+-1.2 libs */
#ifndef USE_GTK_2_0
	#ifdef IS_LINUX
		#undef  __USE_BSD
		#include <string.h>
		#define  __USE_BSD
	#endif
#else
	#include <string.h>
#endif

#ifdef NO_CORE_DUMPS
	#include <sys/prctl.h>
#endif

#define MAX_PATH								PATH_MAX+2
#define MAX_NAME								NAME_MAX+2

/* for checkboxes */
int	flag_nosub=0;
int	flag_disinf=0;
int	flag_rename=0;
int	flag_delete=0;
int	flag_append=0;	
int	flag_list=0;
int	flag_archives=0;
int	flag_passwd_archives=0;
int	flag_comp_exe=0;
int	flag_collect=0;
int	flag_ai=0;
int	flag_follow=0;
int	flag_old=0;

/* for radiobutton groups */
int	flag_scanmode=0;
int	flag_file_types=0;
int	flag_heur=0;
int	flag_macro=0;
int	flag_macro_action=0;

int have_ver_4_4=0; 
int have_verno=0;

GtkWidget	*path_entry_box;
GtkWidget	*report_path_entry_box;
GtkWidget	*action_table;
GtkWidget	*disinfect_button;
GtkWidget	*rename_button;
GtkWidget	*delete_button;
GtkWidget	*macro_remove_table;
GtkWidget	*scan_1_button;
GtkWidget	*scan_2_button;
GtkWidget	*scan_3_button;
GtkWidget	*macro_1_button;
GtkWidget	*macro_2_button;
GtkWidget	*macro_3_button;
GtkWidget	*heur_1_button;
GtkWidget	*heur_2_button;
GtkWidget	*heur_3_button;
GtkWidget	*ai_check_button;
GtkWidget	*archives_button;
GtkWidget	*passwd_archives_button;

/*#################################### File Selection Widget wrappers  ##############################################*/

void do_file_selection(void)
{
	char *path=NULL;
	#ifdef USE_GTK_2_0
	char *filename=NULL;
	#endif

	path=file_selector(STR_SELECT_PATH_TO_SCAN, HIDE_FILE_OP_BUTTONS);

	#ifdef DEBUG
	err_msg("%s() = %s",__FUNCTION__, (path)? path : "NULL");
	#endif

	if(path!=NULL)
	{
		#ifdef USE_GTK_2_0
		if((filename=g_filename_to_utf8(path, -1,NULL,NULL,NULL))!=NULL)
		{
			gtk_entry_set_text(GTK_ENTRY(path_entry_box),filename);
			free(filename);
		}
		#else
		gtk_entry_set_text(GTK_ENTRY(path_entry_box), path);
		#endif
		free(path);
	}
	/*else  path_entry_box unchanged */
}

void do_report_file_selection(void)
{
	char *report_file=NULL;
	#ifdef USE_GTK_2_0
	char *filename=NULL;
	#endif

	report_file=file_selector(STR_SELECT_REPORT_FILE_PATH, HIDE_FILE_OP_BUTTONS);


	#ifdef DEBUG
	err_msg("%s() = %s",__FUNCTION__, (report_file)? report_file : "NULL");
	#endif

	if(report_file!=NULL)
	{
		#ifdef USE_GTK_2_0
		if((filename = g_filename_to_utf8(report_file,  -1,NULL,NULL,NULL))!=NULL)
		{
			gtk_entry_set_text(GTK_ENTRY(report_path_entry_box),filename);
			free(filename);
		}
		#else
		gtk_entry_set_text(GTK_ENTRY(report_path_entry_box), report_file);
		#endif
		free(report_file);
	}
	/*else  report_path_entry_box unchanged */
}

/*############################### End of File Selection Widget wrappers #############################################*/


/*############################### Fprot-info, Virus-list, Update, Test & Reprot button functions ###################*/

void run(char *command_name)
{
	#ifdef DEBUG
	err_msg("%s(): %s",__FUNCTION__, command_name);
	#endif
	system(command_name);
}

void fprot_info()
{
#ifdef OLD_INFO	
	char tmp[sizeof("F-PROT ANTIVIRUS")];
#endif
	
	FILE	*fd;

	DBG_FUNCTION_NAME;

#ifdef OLD_INFO
	/*	This test for older versions of f-prot
		will go away in the future */
	fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");

	if(fgets(tmp,sizeof(tmp),fd)==NULL)
	{
		xpclose_nostdin(fd);
		return;
	}
	
	xpclose_nostdin(fd);

	if(strstr(tmp,"F-PROT ANTIVIRUS")!=NULL)
		SET2(have_verno,1);
#endif

	if(have_verno==1)
	{
		fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
		textview_window_create(STR_INFO,530,430,8,MODAL, fd, NULL);
		xpclose_nostdin(fd);
	}
	else
	{
		/* This is for older versions of f-prot which don't have -verno */
		fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD2,"r");
		textview_window_create(STR_INFO,530,430,8,MODAL, fd, NULL);
		xpclose_nostdin(fd);
	}
}

void fprot_virlist()
{
	FILE	*fd;

	DBG_FUNCTION_NAME;

	fd=xpopen(PATH_TO_F_PROT_BIN" "VIRLIST_CMD,"r");
	textview_window_create(STR_VIRUS_LIST,530,430,8,MODAL, fd, NULL);
	xpclose_nostdin(fd);
}

void fprot_update()
{
	struct	stat stat_p;

	DBG_FUNCTION_NAME;

	if(lstat(UPDATE_CMD_VER_4_1,&stat_p)==0)
	{
		/* For f-prot Program version:>= 4.1.0 */
		#ifdef DEBUG
		err_msg("f-prot version: >= 4.1.0");
		#endif
		run(TERMINAL" su -c "UPDATE_CMD_VER_4_1" &");
	}
	else
	{
		/* For f-prot Program version:< 4.1.0 */
		#ifdef DEBUG
		err_msg("f-prot version: < 4.1.0");
		#endif
		run(TERMINAL" su -c "UPDATE_CMD" &");
	}
}

void fprot_test()
{
	FILE	*fd;

	DBG_FUNCTION_NAME;

	fd=xpopen(PATH_TO_F_PROT_BIN" "VIRTEST_CMD,"r");
	textview_window_create(STR_TEST,530,430,8,MODAL,fd ,NULL);
	xpclose_nostdin(fd);
}

void view_report()
{
	char *report;

	#ifdef USE_OLD_VIEW_REPORT
	FILE	*fd;
	#endif

	DBG_FUNCTION_NAME;

	#ifdef USE_GTK_2_0
	report =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1,NULL,NULL,NULL);
	#else
	report =(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	#endif

	#ifdef USE_OLD_VIEW_REPORT
	if((fd = open_file(report))==NULL)
		return;
	textview_window_create(report,530,430,8,MODAL,fd,NULL);
	xfclose_nostdin(fd);
	#else
	textedit_window_create(report,530,450,8,report);
	#endif
}

/*######################### END of Fprot-info, Virus-list, Update, Test & Report button functions ###################*/

/*############################# START of Scan button function  ######################################################*/

char * add_to_command(char * command, char *option)
{
	int size;
	size=strlen(option)+strlen(command)+1;
	command=(char*)xrealloc(command,size);
	xstrncat(command,option,size);
	#ifdef DEBUG
	err_msg("%s(): %s",__FUNCTION__, option);
	#endif
	return command;
}

void do_scan_f()
{
	int size;
	char * scan_path;
	char * report_path;
	char * fmt="%s %s \"%s\"";
	char * command;

	DBG_FUNCTION_NAME;

	#ifdef USE_GTK_2_0
	scan_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(path_entry_box)), -1, NULL, NULL, NULL);
	#else
	scan_path =(char *) gtk_entry_get_text(GTK_ENTRY(path_entry_box));
	#endif

	size = strlen(TERMINAL)+ strlen(PATH_TO_F_PROT_BIN) + strlen(scan_path)+strlen(fmt)+1;
	command=(char *)xmalloc(size);

	/* We always "" quote the path to scan to avoid troubles with filenames with spaces
		which are split by the shell and mess up our command line args */

	xsnprintf(command, size, fmt, TERMINAL, PATH_TO_F_PROT_BIN, scan_path);

	/* I prefer to create always a report file so no flag checking here.*/
	/* BTW we always quote filenames and paths */
	command=add_to_command(command, " -report=");
	command=add_to_command(command, "\"");

	#ifdef USE_GTK_2_0
	report_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);
	#else
	report_path=(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	#endif

	command=add_to_command(command, report_path);
	command=add_to_command(command, "\"");

	/* Let's parse the options*/
	if(flag_scanmode > 0 ) /* not in report only mode */
	{
		if(IS_SET(flag_disinf))
			command=add_to_command(command," -disinf");
		if(IS_SET(flag_rename))
			command=add_to_command(command," -rename");
		if(IS_SET(flag_delete))
			command=add_to_command(command," -delete");
		
		/* flag_scanmode == 1 is the default: Interactive */
		if(flag_scanmode==2) /* Automatic */
			command=add_to_command(command," -auto");
	}

	if(flag_file_types==0)
		command=add_to_command(command," -dumb");	/* "Scan all files" this is the default */
	else if(flag_file_types==1)
		command=add_to_command(command," -ext");	/* "Standard file extensions" */
	else if(flag_file_types==2)
		command=add_to_command(command," -type");	/* "Attempt to identify files" */
	#ifdef DEBUG
	else
	{
		err_msg("Bad value: flag_file_types");
		exit(EXIT_FAILURE);
	}
	#endif	
				
	if(IS_SET(flag_archives))
	{
		command=add_to_command(command," -archive");
		if(IS_SET(have_ver_4_4))
		{
			/* Attempts to identify infections within password protected archives. The option -server implies -archive.*/
			if(IS_SET(flag_passwd_archives))
				command=add_to_command(command," -server");
			else
				command=add_to_command(command," -noserver");	
		}
	}
	else
	{
		if(IS_SET(have_ver_4_4))
		{
			/*Does not scan inside archives. The option -noarchive implies -noserver.*/
			command=add_to_command(command," -noarchive");
			command=add_to_command(command," -noserver");	
		}
	}
	 /* Unpack compressed executables. There is no corresponding -nopacked option. 
	 This option is provided for legacy reasons. */
	if(IS_SET(flag_comp_exe))
		command=add_to_command(command," -packed");
	if(IS_SET(flag_nosub))
		command=add_to_command(command," -nosub");
	if(IS_SET(flag_collect))
		command=add_to_command(command," -collect");
	if(IS_SET(flag_list))
		command=add_to_command(command," -list");
	if(IS_SET(flag_ai))
		command=add_to_command(command," -ai");
	if(IS_SET(flag_old))
		command=add_to_command(command," -old");
	/* "Follow symbolic links" is not displayed in f-prot switches but works correctly	*/
	if(IS_SET(flag_follow))
		command=add_to_command(command," -follow");
	if(IS_SET(flag_append))
		command=add_to_command(command," -append");

	if(flag_heur==1)
		command=add_to_command(command," -noheur");
	else if (flag_heur==2)
		command=add_to_command(command," -onlyheur");
	#ifdef DEBUG
	/* flag_heur == 0 is default */
	else if (flag_heur == 0)
		/* VOID */;
	else
	{
		err_msg("Bad value: flag_heur");
		exit(EXIT_FAILURE);
	}
	#endif	

	if(flag_macro==1)
		command=add_to_command(command," -nomacro");
	else if (flag_macro==2)
		command=add_to_command(command," -onlymacro");
	#ifdef DEBUG
	/* flag_macro == 0 is default */
	else if (flag_macro == 0)
		/* VOID */;
	else
	{
		err_msg("Bad value: flag_macro");
		exit(EXIT_FAILURE);
	}
	#endif	
	
	if(flag_scanmode > 0 && flag_macro > 0 )
	{
		if(flag_macro_action==0)
			command=add_to_command(command," -saferemove");	/*"Remove infected macros from docs"*/
		else if (flag_macro_action==1)
			command=add_to_command(command," -removenew");	/*"Remove all macros from infected docs"*/
		else if (flag_macro_action==2)
			command=add_to_command(command," -removeall");	/*"Remove all macros from all docs"*/
		#ifdef DEBUG
		else
		{
			err_msg("Bad value: flag_macro_action");
			exit(EXIT_FAILURE);
		}
		#endif	
	}
	/*  -wrap switch 	is not supported, if you want to edit the logs use the text editor */
	/*  -nobreak    	is not supported (Do not abort scan if ESC is pressed) */
	/*  -page       	is not supported (Pause after each page.)*/
	/*  -silent     	is not supported (Do not generate any screen output.)*/

	command=add_to_command(command, " &");

	msg("executing scan on: %s", scan_path);
	run(command);
	free(command);
}

void scan_f()
{
	char *report_path;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;

	/* Check if at least a flag for actions is set, if not make sure that scanmode is set to report only */
	if((IS_NOT_SET(flag_disinf) && IS_NOT_SET(flag_rename) && IS_NOT_SET(flag_delete)) && flag_scanmode > 0 )
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button),TRUE);
		SET2(flag_scanmode,0);
		dialog_window(STR_ERROR, STR_NO_ACTION_DOING_REPORT_ONLY, ONE_BUTTON, MODAL, NULL);
	}
	
	#ifdef USE_GTK_2_0
	report_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);
	#else
	report_path=(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	#endif

	if(lstat(report_path,&stat_p)==0) /* File exists */
	{
		if(!S_ISREG(stat_p.st_mode)) /* is not a regular file */
		{
			dialog_window(STR_ERROR, STR_REPORT_FILE_IS_NOT_A_REGULAR_FILE ,ONE_BUTTON, MODAL, NULL);
			return;
		}
		else /* is a regular file */
		{
			if(IS_NOT_SET(flag_append))  /* Don't append */
			{
				if(dialog_window(STR_WARNING, STR_REPORT_FILE_EXISTS_OVERWRITE, TWO_BUTTONS, MODAL, NULL)==1)
				{
					/* Don't overwrite */
					return;
				}
				/*else overwrite existing report */
			}
			/*else append to existing report */
		}
	}
	/*else file don't exist so write it now */
	do_scan_f();
}

/*############################## End of Scan button function  ######################################################*/

void remove_lock()
{
	DBG_FUNCTION_NAME;

	if(remove(LOCK_FILE)==-1)
	{
		err_msg("failed to remove lock file: %m");
		dialog_window(STR_ERROR, STR_FAILED_TO_REMOVE_LOCK_FILE_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
	msg("removed lock file");
}

void about_f()
{
	FILE	*fd;

	fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
	about_window_create(VERSION, 620, 430, 8, MODAL,VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
						fd,NOT_FIRST_TIME);
	xfclose_nostdin(fd);
}

void create_file(char * filename)
{
	int pfd;

	if ((pfd=open(filename ,O_RDWR|O_CREAT, 0600))!=-1)
			close(pfd);
	#ifdef DEBUG
	else
		err_msg("cannot open file: %s: %m", filename); /* This error is ignored */
	#endif
}

void root_warn()
{
	int ret;
	struct stat stat_p;

	DBG_FUNCTION_NAME;

	msg("you are root!");

	if(lstat(NO_ROOT_WARN_FILE,&stat_p)!=0)
	{
		ret=dialog_window(VERSION" - "STR_WARNING, STR_YOU_ARE_RUNNING_XFPROT_AS_ROOT_CONTINUE,
						TWO_BUTTONS_ONE_CHECK, MODAL, STR_DONT_SHOW_IT_AGAIN);
		switch(ret)
		{
			case 4: /* No + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
				/* FALLTROUGH */
			case 1: /* No */
				remove_lock();
				msg("...exiting");
				exit(EXIT_SUCCESS);
			case 0: /* Ok */
				break;
			case 3: /* Ok + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
				break;
			/*esac */
		}
	}
	#ifdef DEBUG
	else
		err_msg("NO_ROOT_WARN_FILE");
	#endif
}

void do_lock_file()
{
	int pfd;
	int dialog_retval=-1;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;

	/* Check if LOCK FILE exists and remove if stale */
	if(lstat(LOCK_FILE,&stat_p)==0) /* File exists */
	{
		err_msg("lock file found");

		dialog_retval = dialog_window(VERSION" - "STR_WARNING, STR_XFPROT_HAS_DETECTED_A_LOCK_FILE,TWO_BUTTONS, MODAL, NULL);
		if(dialog_retval==1)
		{
			msg("...exiting");
			exit(EXIT_SUCCESS);
		}
		else
			remove_lock();
	}

	if ((pfd=open(LOCK_FILE,O_RDWR|O_CREAT|O_EXCL,0640))<0) /* cannot open */
	{
		err_msg("cannot open lock file: %m");
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_OPEN_LOCK_FILE_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
	if (lockf(pfd,F_TLOCK,0)<0) /* cannot lock */
	{
		err_msg("lockf(): %m");
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_LOCK_FILE_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
	/* only first instance continues */
	close(pfd);
}

void set_dir_properties_and_cd_in(uid_t uid, gid_t gid)
{
	DBG_FUNCTION_NAME;

	msg("setting permissions of private dir to 0700");
	if(xchmod(XFPROT_PRIV_DIR,0700, RETURN_ON_FAILURE)==-1)
	{
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_SET_PERMISSIONS_OF_PRIVATE_DIR_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}

	msg("setting owner of private dir");
	if(	xchown(XFPROT_PRIV_DIR, uid, gid, RETURN_ON_FAILURE)==-1)
	{
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_SET_OWNER_OF_PRIVATE_DIR_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}

	msg("chdir to private dir");
	if(	xchdir(XFPROT_PRIV_DIR, RETURN_ON_FAILURE)==-1)
	{
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_CHANGE_DIR_TO_PRIVATE_DIR_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
}


void check_for_bin()
{
	DBG_FUNCTION_NAME;

	/* access  may  not  work  correctly  on NFS file systems */
	if (access(PATH_TO_F_PROT_BIN , X_OK)<0)
	{
		err_msg("access(): %s: %m",PATH_TO_F_PROT_BIN);
		dialog_window(VERSION" - "STR_ERROR, PATH_TO_F_PROT_BIN"\n"STR_FILE_NOT_FOUND_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
}

void toggle1_f()
{
	DBG_FUNCTION_NAME;

	if(flag_scanmode > 0 && flag_macro < 2 )
	{
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(action_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disinfect_button),	TRUE);
		SET(flag_disinf);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rename_button),	FALSE);
		RESET(flag_rename);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (delete_button),	FALSE);
		RESET(flag_delete);
	}
	else
	{
		gtk_widget_hide(action_table);
		#ifdef DEBUG
		err_msg("hiding widget action_table");
		#endif
	}
}

void toggle2_f()
{
	DBG_FUNCTION_NAME;

	if( flag_scanmode > 0 && flag_macro != 1 )
	{
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(macro_remove_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_1_button), TRUE);
		SET2(flag_macro_action,0);
	}
	else
	{
		gtk_widget_hide(macro_remove_table);
		#ifdef DEBUG
		err_msg("hiding widget macro_remove_table");
		#endif
	}
}

/* This callback quits the program */
void delete_event( void )
{
	DBG_FUNCTION_NAME;

	remove_lock();
	gtk_main_quit ();
	msg("...exiting");
	exit(EXIT_SUCCESS);
}

void flag_scanmode_callback( GtkWidget *widget, gpointer callback_data )
{
	exit_if_null(widget);

	SET2(flag_scanmode,atoi(callback_data));
	toggle1_f();
	toggle2_f();
}

void flag_file_types_callback(GtkWidget *widget, gpointer callback_data )
{
	exit_if_null(widget);

	SET2(flag_file_types,atoi(callback_data));
}

void flag_macro_callback(GtkWidget *widget, gpointer callback_data )
{
	exit_if_null(widget);

	SET2(flag_macro,atoi(callback_data));
	toggle2_f();
	toggle1_f();
}

void flag_macro_action_callback(GtkWidget *widget, gpointer callback_data )
{
	exit_if_null(widget);

	SET2(flag_macro_action,atoi(callback_data));
}

void flag_heur_callback(GtkWidget *widget, gpointer callback_data )
{
	exit_if_null(widget);

	SET2(flag_heur,atoi(callback_data));
	/* Disable neural-network virus detection. The -ai option should not be used with the -noheur option.*/
	if(flag_heur==1)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button),FALSE);
}

void flag_ai_callback(GtkWidget *widget, int *callback_data )
{
	exit_if_null(widget);

	if(IS_SET(*callback_data))
		RESET(*callback_data);
	else
	{
		/* Enable heuristics.The -ai option (neural-network virus detection) should not be used with the -noheur option.*/
		if(flag_heur==1)
		{
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_1_button), TRUE);
			SET2(flag_heur,0);
		}
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button),TRUE);
		SET(*callback_data);	
	}
}

void flag_archives_callback(GtkWidget *widget, int *callback_data )
{
	exit_if_null(widget);

	if(IS_SET(*callback_data))
	{
		RESET(*callback_data);
		if(IS_SET(have_ver_4_4))
		{
			/* check for flag_protected_archives and disable if set */
			if(IS_SET(flag_passwd_archives))
			{
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (passwd_archives_button),FALSE);
				SET2(flag_passwd_archives,0);
			}
		}
	}
	else
		SET(*callback_data);	
}

void flag_passwd_archives_callback(GtkWidget *widget, int *callback_data )
{
	exit_if_null(widget);
	
	if(IS_SET(have_ver_4_4))
	{
		if(IS_SET(*callback_data))
			RESET(*callback_data);
		else
		{
			/* check for flag_archives and enable if unset */
			if(IS_NOT_SET(flag_archives))
			{
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (archives_button),TRUE);
				SET2(flag_archives,1);
			}
			SET(*callback_data);	
		}
	}
}

void flag_callback(GtkWidget *widget, int *callback_data )
{
	exit_if_null(widget);

	if(IS_SET(*callback_data))
		RESET(*callback_data);
	else
		SET(*callback_data);	
}

void signal_handler( int sig)
{
	switch(sig)
	{
		case SIGINT:
				msg("SIGINT signal catched");
				delete_event();
				break;
		case SIGTERM:
				msg("SIGTERM signal catched");
				delete_event();
				break;
	}
}

int main(int argc, char **argv)
{
	GtkWidget		*main_window;
	GtkWidget		*label;
	GtkWidget		*top_table;
	GtkWidget		*mode_table;
	GtkWidget		*macro_table;
	GtkWidget		*file_types_table;
	GtkWidget		*report_table;
	GtkWidget		*option_table;
	GtkWidget		*frame;
	GtkWidget		*table;
	GtkWidget		*button;
	GtkWidget		*radio_button;
	GtkWidget		*check_button;
	GtkTooltips		*action_tooltip;
	GtkTooltips		*onlyheur_tooltip;
	GSList			*scan_mode_group=NULL;
	GSList			*scan_macro_group=NULL;
	GSList			*file_types_group=NULL;
	GSList			*remove_macro_group=NULL;
	GSList			*heuristics_group=NULL;

	int	*ptr_flag_nosub;
	int	*ptr_flag_disinf;
	int	*ptr_flag_rename;
	int	*ptr_flag_delete;
	int	*ptr_flag_append;	
	int	*ptr_flag_list;
	int	*ptr_flag_archives;
	int *ptr_flag_passwd_archives;
	int	*ptr_flag_comp_exe;
	int	*ptr_flag_collect;
	int	*ptr_flag_ai;
	int	*ptr_flag_follow;
	int	*ptr_flag_old;
	
	char tmp[64];
	char *path=NULL;
	char *report_file_path=NULL;
	FILE	*fd;


	struct	passwd *pw;
	struct	stat stat_p;

	/* print disclaimer to terminal */
	xputs("\n"VERSION"\n\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY"\n\n");
	msg("starting...");

	signal(SIGHUP,SIG_IGN);  			/* ignore SIGHUP */
	signal(SIGTERM,signal_handler);		/* catch SIGTERM */
	signal(SIGINT,signal_handler);		/* catch SIGINT  */

	gtk_set_locale();
	gtk_init (&argc, &argv);

	/* Force the output to be unbuffered when debugging.*/
	#ifdef DEBUG
	setbuf(stdout, (char *) 0);
	#endif

	/* Command line args. No dialog windows on errors as this is console stuff */
	if( argc == 2)
	{
		if(strcmp(argv[1],"-v")==0)
		{
			msg(VERSION);
			exit(EXIT_SUCCESS);
		}
		err_msg("unknown command line switch: %s",argv[1]);
		exit(EXIT_FAILURE);
	}
	if(argc > 2)
	{
		err_msg("too much args on command line");
		exit(EXIT_FAILURE);
	}

	#ifdef NO_CORE_DUMPS
	/* make the process not dumpable under Linux 2.4, seems to work also for 2.6.0-test3 */
	prctl(PR_SET_DUMPABLE,0);
	msg("process is%sdumpable",(prctl(PR_GET_DUMPABLE)==0)?" not ":" ");
	#endif

	/* Try to determine the home dir of the user in
		passwd struct to avoid use of getenv("HOME").
		Create a private xfprot dir in user's home dir
		if it doesn't already exist.
		Set explicitly ownership and permissions of xfprot's dir.
		If something goes wrong we issue an error and exit.
		BTW: the user is forced to write the report always in
		xfprot's  private dir.
	*/

	/* Who am I ? */
	if((pw=getpwuid(getuid()))==NULL)
	{
		#ifdef DEBUG
		err_msg("getpwuid(getuid()): %m");
		#endif
		dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_GET_PWENT_ABORTING, ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
	/* chdir to user's home dir */
	if((xchdir(pw->pw_dir, RETURN_ON_FAILURE))==EXIT_ERROR)
	{
		dialog_window(VERSION" - "STR_ALERT, STR_FAILED_TO_CD_TO_HOMEDIR_TRYING_ROOTDIR, ONE_BUTTON, MODAL, NULL);
		if(xchdir("/",RETURN_ON_FAILURE)==-1)
		{
			dialog_window(VERSION" - "STR_ERROR, STR_FAILED_TO_CD_TO_ROOTDIR_ABORTING, ONE_BUTTON, MODAL, NULL);
			exit(EXIT_FAILURE);
		}
	}
	/* Check for xfprot's private dir */
	if(lstat(XFPROT_PRIV_DIR,&stat_p)==0)
	{
		msg("private dir found");
		if(S_ISLNK(stat_p.st_mode))
		{
			/* We don't want to be redirected by a link */
			#ifdef DEBUG
			err_msg("private dir is a link");
			#endif
			dialog_window(VERSION" - "STR_ERROR, STR_PRIVATE_DIR_IS_A_LINK_ABORTING, ONE_BUTTON, MODAL, NULL);
			exit(EXIT_FAILURE);
		}
		#ifdef DEBUG
		if (S_ISDIR(stat_p.st_mode))
		{
			/* OK let's go on */
			err_msg("good...private dir is not a link");
		}
		#endif
	}
	else
	{
		err_msg("private dir not found");
		if(xmkdir(XFPROT_PRIV_DIR, 0700, RETURN_ON_FAILURE)==-1)
		{
			dialog_window(VERSION" - "STR_ERROR, STR_CANNOT_CREATE_PRIVATE_DIR_ABORTING, ONE_BUTTON, MODAL, NULL);
			exit(EXIT_FAILURE);
		}
	}

	/* Explicitly set private dir's permissions and owner and chdir into it */
	set_dir_properties_and_cd_in( pw->pw_uid, pw->pw_gid);
	/* check for f-prot*/
	check_for_bin();
	/* lock file stuff */
	do_lock_file();
	/* display a warning if we are root */
	if(pw->pw_uid==0 )
		root_warn();
	/* display license window, this can be disabled */
	if(lstat(NO_SPLASH_FILE,&stat_p)!=0)
	{
		fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
		if(about_window_create(VERSION, 620, 430,8,MODAL,VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
								fd,FIRST_TIME)==1)
			create_file(NO_SPLASH_FILE);
		xfclose_nostdin(fd);
	}
	#ifdef DEBUG
	else
		err_msg("NO_SPLASH_FILE");
	#endif

	/* Detect f-prot version 4.4.x for -server/-noserver switch and detect -verno switch */
	fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
	while(fgets(tmp,sizeof(tmp),fd)!=NULL)
	{
		if(strstr(tmp,"F-PROT ANTIVIRUS")!=NULL)
			SET2(have_verno,1);
		if(strstr(tmp,"Program version: 4.4")!=NULL)
			SET(have_ver_4_4);
	}
	xpclose_nostdin(fd);

	/* Initialize our flags */	
	ptr_flag_nosub	=	&flag_nosub;
	ptr_flag_disinf	=	&flag_disinf;
	ptr_flag_rename	=	&flag_rename;
	ptr_flag_delete	=	&flag_delete;
	ptr_flag_append	=	&flag_append;	
	ptr_flag_list	=	&flag_list;
	ptr_flag_archives =	&flag_archives;
	ptr_flag_passwd_archives = &flag_passwd_archives;
	ptr_flag_comp_exe =	&flag_comp_exe;
	ptr_flag_collect =	&flag_collect;
	ptr_flag_ai		=	&flag_ai;
	ptr_flag_follow =	&flag_follow;
	ptr_flag_old	=	&flag_old;
	
	main_window = window_create(640,  450, 0, VERSION, delete_event, NULL, NOT_MODAL);
	table = table_create(10, 20, main_window, 2 ,  NOT_HOMOGENEUS);

	frame = frame_create_on_table(STR_PATH_TO_SCAN, table, 0, 10, 0,3);
	top_table = table_create(20, 2, frame, 2 , HOMOGENEUS);
	path=xgetcwd(path);

	#ifdef DEBUG
	err_msg(path);
	#endif

	path_entry_box = text_box_create_on_table(path, NOT_EDITABLE, top_table, 0, 19, 0, 1);

	button = button_create_on_table(":",GTK_SIGNAL_FUNC(do_file_selection), NULL, top_table, 19, 20, 0,1);

	check_button = check_create_on_table(STR_NOSUB, GTK_SIGNAL_FUNC(flag_callback), ptr_flag_nosub, 
										INACTIVE, top_table, 0, 6, 1, 2);

	frame = frame_create_on_table(STR_SCAN_MODE, table, 0, 5, 3, 7);
	mode_table = table_create(1, 3, frame, 2 , HOMOGENEUS);

	/* Warning we can't use the ACTIVE flag here 'cause the callback function acts on widgets
		we haven't created yet, so we do things manually later by callin toggle1_f() */
	scan_1_button = radio_create_on_table(NULL,STR_REPORT_ONLY,GTK_SIGNAL_FUNC(flag_scanmode_callback),"0",
													INACTIVE,mode_table, 0, 1, 0,1);

	/* Maybe this is not needed as first radio button is set active by default, but to be sure.....*/
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button),TRUE);

	scan_mode_group = get_radio_button_group (GTK_RADIO_BUTTON (scan_1_button));
	scan_2_button = radio_create_on_table(scan_1_button,STR_INTERACTIVE,GTK_SIGNAL_FUNC(flag_scanmode_callback),"1",
													INACTIVE,mode_table, 0, 1, 1,2);
	scan_3_button = radio_create_on_table(scan_2_button, STR_AUTOMATIC,GTK_SIGNAL_FUNC(flag_scanmode_callback),"2",
													INACTIVE,mode_table, 0, 1, 2,3);

	frame = frame_create_on_table( STR_SCAN_FOR_MACRO_VIRUS, table, 5, 10, 3, 7);
	macro_table = table_create(1, 3, frame, 2 ,  HOMOGENEUS);

	/* Warning we can't use the ACTIVE flag here because the callback function acts on widgets
		we haven't created yet, so we do things manually */
	radio_button = radio_create_on_table(NULL, STR_YES, GTK_SIGNAL_FUNC(flag_macro_callback),
										"0", INACTIVE, macro_table, 0, 1, 0, 1);

	/* Maybe this is not needed as first radio button is set active by default, but to be sure.....*/
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button),TRUE);

	scan_macro_group = get_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = radio_create_on_table(radio_button,STR_NO,GTK_SIGNAL_FUNC(flag_macro_callback),
										"1", INACTIVE, macro_table, 0, 1, 1, 2);
	radio_button = radio_create_on_table(radio_button,STR_ONLY,GTK_SIGNAL_FUNC(flag_macro_callback),
										"2", INACTIVE, macro_table, 0, 1, 2, 3);
	
	frame = frame_create_on_table( STR_ACTIONS_TO_TRY_ON_INFECTED_FILES, table, 0, 5, 7, 11);
	action_table = table_create(1, 3, frame, 2 ,  HOMOGENEUS);
	action_tooltip=gtk_tooltips_new();

	#ifndef USE_GTK_2_0
	/* Warning: Gtk+ 1.2.10 docs for gtk_tooltips_set_delay say:
	 * delay: an integer value representing seconds !!!
	 * but in gtktooltips.c  we read #define DEFAULT_DELAY 500   (Default delay in ms)
	 * So this seems to works for me only using seconds*1000
	 */
	gtk_tooltips_set_delay (action_tooltip,2000); /* 2 seconds */
	#endif

	disinfect_button =check_create_on_table(	STR_DISINFECT_WHENEVER_POSSIBLE,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_disinf,
												ACTIVE,action_table, 0, 1, 0, 1);

	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),disinfect_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);

	rename_button =		check_create_on_table(	STR_RENAME_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_rename,
												INACTIVE, action_table, 0, 1, 1, 2);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),rename_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);
	delete_button =		check_create_on_table(	STR_DELETE_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_delete,
												INACTIVE, action_table, 0, 1, 2, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),delete_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);
	gtk_tooltips_enable (action_tooltip);

	frame = frame_create_on_table(STR_MACRO_VIRUS_REMOVAL_OPTIONS, table, 5, 10, 7, 11);
	macro_remove_table = table_create(1, 3, frame, 2 ,  HOMOGENEUS);
	macro_1_button = radio_create_on_table(	NULL, STR_REMOVE_INFECTED_MACROS_FROM_DOCS,
											GTK_SIGNAL_FUNC(flag_macro_action_callback), "0",
											ACTIVE, macro_remove_table, 0, 1, 0, 1);
	remove_macro_group = get_radio_button_group (GTK_RADIO_BUTTON (macro_1_button));
	macro_2_button = radio_create_on_table(	macro_1_button,STR_REMOVE_ALL_MACROS_FROM_INFECTED_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "1",
													INACTIVE, macro_remove_table, 0, 1, 1, 2);
	macro_3_button = radio_create_on_table(	macro_2_button,STR_REMOVE_ALL_MACROS_FROM_ALL_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "2",
													INACTIVE, macro_remove_table, 0, 1, 2, 3);

	frame = frame_create_on_table(STR_FILE_TYPES, table, 0, 5, 11, 15);
	file_types_table = table_create(1, 3, frame, 2 ,  HOMOGENEUS);

	radio_button = radio_create_on_table(NULL,	STR_SCAN_ALL_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"0",
										ACTIVE,file_types_table, 0, 1, 0, 1);
	file_types_group = get_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = radio_create_on_table(radio_button,STR_STANDARD_FILE_EXTENSIONS,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"1",
										INACTIVE,file_types_table, 0, 1, 1, 2);
	radio_button = radio_create_on_table(radio_button,STR_ATTEMPT_TO_IDENTIFY_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"2",
										INACTIVE,file_types_table, 0, 1, 2, 3);
										
	frame = frame_create_on_table(STR_REPORT_FILE, table, 0, 5, 15, 19);
	report_table = table_create(8, 3, frame, 2 , HOMOGENEUS);


	report_file_path=xconcat_path_file(path, DEF_REPORT_FILE);

	report_path_entry_box = text_box_create_on_table( report_file_path , NOT_EDITABLE, report_table, 0, 7, 0, 1);
	button = button_create_on_table(":"			,GTK_SIGNAL_FUNC(do_report_file_selection)	, NULL, report_table, 7, 8, 0, 1);

	free(report_file_path);
	free(path);

	check_button = check_create_on_table(STR_APPEND_TO_EXISTING_REPORT_FILE,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_append,
										INACTIVE, report_table, 0, 7, 1, 2);
	check_button = check_create_on_table( STR_LIST_ALL_SCANNED_FILES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_list,
										INACTIVE, report_table, 0, 7, 2, 3);

	frame = frame_create_on_table(STR_OPTIONS, table, 5, 10, 11, 19);
	option_table = table_create(5, 7, frame, 2 , HOMOGENEUS);

	archives_button = check_create_on_table(STR_SCAN_INSIDE_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_archives_callback), ptr_flag_archives,
										ACTIVE, option_table, 0, 5, 0, 1);
	

	if(IS_NOT_SET(have_ver_4_4))
	{
		check_button = check_create_on_table(STR_SCAN_COMPRESSED_EXECUTABLES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_comp_exe,
										ACTIVE, option_table, 0, 5, 1, 2);
	}
	else
	{
		/* -packed is default in f-prot 4.4.1 so we don't need a checkbox for it
		 as a nopacked options doesn't exist and we can use this place for this new option */
		passwd_archives_button = check_create_on_table( STR_SCAN_PROTECTED_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_passwd_archives_callback), ptr_flag_passwd_archives,
										INACTIVE, option_table, 0, 5, 1, 2);
	}
	
	check_button = check_create_on_table(STR_SCAN_FOR_OTHER_VARIOUS_MALWARE,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_collect,
										INACTIVE, option_table, 0, 5, 2, 3);
	ai_check_button = check_create_on_table(STR_ENABLE_NEURAL_NETWORK_VIRUS_DETECTION,
										GTK_SIGNAL_FUNC(flag_ai_callback), ptr_flag_ai,
										INACTIVE, option_table, 0, 5, 3, 4);
	check_button = check_create_on_table(STR_FOLLOW_SYMBOLIC_LINKS,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_follow,
										INACTIVE, option_table, 0, 5, 4, 5);
	check_button = check_create_on_table(STR_DO_NOT_COMPLAIN_ABOUT_OLD_DEF_FILES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_old,
										INACTIVE, option_table, 0, 5, 5, 6);

	label = label_create_on_table( STR_USE_HEURISTICS, option_table, 0, 2, 6,7);
	heur_1_button = radio_create_on_table(NULL, STR_YES,GTK_SIGNAL_FUNC(flag_heur_callback), "0",
										 ACTIVE, option_table, 2, 3, 6, 7);
	heuristics_group = get_radio_button_group (GTK_RADIO_BUTTON (heur_1_button));
	heur_2_button = radio_create_on_table(heur_1_button, STR_NO,GTK_SIGNAL_FUNC(flag_heur_callback), "1",
										INACTIVE, option_table, 3, 4, 6, 7);
	heur_3_button = radio_create_on_table(heur_2_button, STR_ONLY,GTK_SIGNAL_FUNC(flag_heur_callback), "2",
										INACTIVE, 	option_table, 4, 5, 6, 7);
	
	onlyheur_tooltip=gtk_tooltips_new();
	#ifndef USE_GTK_2_0
	/* Warning: Gtk+ 1.2.10 docs for gtk_tooltips_set_delay say:
	 * delay: an integer value representing seconds !!!
	 * but in gtktooltips.c  we read #define DEFAULT_DELAY 500   (Default delay in ms)
	 * So this seems to works for me only using seconds*1000
	 */
	gtk_tooltips_set_delay (onlyheur_tooltip,2000); /* 2 seconds */
	#endif
	gtk_tooltips_set_tip (GTK_TOOLTIPS (onlyheur_tooltip),heur_3_button,STR_NOHEUR_TOOLTIP,NULL);
	
	button = button_create_on_table(STR_EXIT		,GTK_SIGNAL_FUNC(delete_event)	, NULL, table, 9, 10, 19, 20);
	button = button_create_on_table(STR_INFO		,GTK_SIGNAL_FUNC(fprot_info)	, NULL, table, 7, 8,  19, 20);
	button = button_create_on_table(STR_VIRUS_LIST	,GTK_SIGNAL_FUNC(fprot_virlist)	, NULL, table, 6, 7,  19, 20);
	button = button_create_on_table(STR_TEST		,GTK_SIGNAL_FUNC(fprot_test)	, NULL, table, 5, 6,  19, 20);
	button = button_create_on_table(STR_UPDATE		,GTK_SIGNAL_FUNC(fprot_update)	, NULL, table, 4, 5,  19, 20);
	button = button_create_on_table(STR_REPORT		,GTK_SIGNAL_FUNC(view_report)	, NULL, table, 3, 4,  19, 20);
	button = button_create_on_table(STR_SCAN		,GTK_SIGNAL_FUNC(scan_f)		, NULL, table, 2, 3,  19, 20);
	button = button_create_on_table(STR_ABOUT		,GTK_SIGNAL_FUNC(about_f)		, NULL, table, 0, 1,  19, 20);

	gtk_widget_show_all(main_window);
	toggle1_f();
	toggle2_f();
	gtk_main();

	return(EXIT_SUCCESS);
}
